#include <math.h>
#include <algorithm>

#include "./ball.h"
#include "./common.h"

using namespace std;

const double acc_f = -0.0002;

void Ball::draw() const {
  // glColor3f(0.0, 0.0, 0.0);
  glColor4fv(_color.get());
  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
  glBegin(GL_POLYGON);

  static const int inc = 5;
  for (int i = 0; i <= 360; i+=inc) {
    const double theta = i*PI/180.0;
    const point2 p(_pos.x() + _r * cos(theta), _pos.y() + _r * sin(theta));
    glVertex2f(p.x(), p.y());
  }

  glEnd();
}

void Ball::accelerate(const vec2& a, const vec2& v) {
  _a = v.normalized() * acc_f;
  _v = v;
  _t0 = glutGet(GLUT_ELAPSED_TIME);
  _t = glutGet(GLUT_ELAPSED_TIME);
}

void Ball::set_velocity(const vec2& v) {
  _a = v.normalized() * acc_f;
  _v = v;
  _t0 = glutGet(GLUT_ELAPSED_TIME);
  _t = glutGet(GLUT_ELAPSED_TIME);
}

namespace {
template <typename T>
int sign(T v) {
  if (v < 0) return -1;
  if (v > 0) return 1;
  return 0;
}
};

void Ball::update() {
  const float t = glutGet(GLUT_ELAPSED_TIME);
  const float time_tot = t - _t0;
  const float time_h = t - _t;
  vec2 v(_a.x() * time_h + _v.x(), _a.y() * time_h + _v.y());
  if (sign(v.x()) != sign(_v.x())) {
    v = vec2();
    _a = vec2();
  }
  const vec2 d(v.x() * time_h, v.y() * time_h);
  _pos = _pos + d;

  _t = t;
  _v = v;
}

bool Ball::update(Ball* ball) {
  const float dist = (pos()-ball->pos()).length();
  if (dist > _r + ball->_r) return false;

  // intersection - first bounce this off b
  const point2 a = pos();
  const point2 b = ball->pos();
  const vec2 u = b-a;
  const point2 c = a + _r * u.normalized();
  const vec2 v = vec2(u.y(), -u.x());
  const point2 d = c + v;
  const vec2 r = reflection(vec(), c, d);
  const float energy = vec().length();
  set_velocity((energy/2) * r.normalized());
  _pos = _last_good_pos;

  ball->set_velocity((energy/2) * u.normalized());
  ball->_pos = ball->_last_good_pos;

  return true;
}
