#ifndef __BALL_H__
#define __BALL_H__

#include "./geom2.h"
#include "./color.h"

// A handy Ball class
class Ball {
 public:
  explicit Ball(const point2& pos, const color& c)
      : _r(10), _a(0, 0), _v(0, 0), _pos(pos), _last_good_pos(pos),
        _color(c) {}
  Ball() : _r(10), _a(0, 0), _v(0, 0), _pos(0, 0) {}
  const point2& pos() const { return _pos; }
  float radius() const { return _r; }
  void revert_pos() { _pos = _last_good_pos; }
  const point2& last_good_pos() const { return _last_good_pos; }
  void update_last_good_pos() { _last_good_pos = _pos; }
  const vec2& vec() const { return _v; }
  void draw() const;
  void accelerate(const vec2& a, const vec2& v);
  void set_velocity(const vec2& v);
  void update();
  bool active() const { return _v.x() != 0 && _v.y() != 0; }

  bool update(Ball* b);

 private:
  // radius
  float _r;
  // acceleration
  vec2 _a;
  // velocity
  vec2 _v;
  // position
  point2 _pos;
  // last good position
  point2 _last_good_pos;
  // time of the last acceleration change
  float _t0;
  // time of the last position update
  float _t;

  // color
  color _color;
};


#endif
