#include <math.h>
#include <iostream>
#include <utility>
#include <vector>

#include "./common.h"
#include "./ball.h"

void Display1();
void MouseMotion1(int x, int y);
void Display2();
void MouseMotion2(int x, int y);
void Display3();
void MouseMotion3(int x, int y);
void Display4();
void MouseMotion4(int x, int y);
void Display5();
void MouseMotion5(int x, int y);
void Display6();
void MouseMotion6(int x, int y);

using namespace std;

int window_width = 500;
int window_height = 500;
bb2 bb(-100, -100, 200, 200);

point2 mouse_down;

// acceleration factor
const double vel_f = 0.004;

bool down = false;
point2 mouse_cur;

float sq(float f) {
  return f*f;
}

Ball ball(point2(-50, -50), color(1.0f, 1.0f, 1.0f));
Ball ball2(point2(), color(1.0f, 0.0f, 0.0f));

void hit(const point2& P, const point2& Q) {
  vec2 v = Q-P;
  const float len = v.length();
  v = v / len;
  // const vec2 acc = v * acc_f;
  const vec2 vel = v * len * vel_f;
  // ball.accelerate(acc, vel);
  ball.set_velocity(vel);
}

point2 win2obj(const point2& p) {
  return point2(p.x()/static_cast<float>(window_width) * 200 - 100,
                p.y()/static_cast<float>(window_height) * 200 - 100);
}

std::vector<Ball> balls;
extern int num_balls;

void init() {
  glClearColor(0.0, 0.7, 0.0, 1.0);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluOrtho2D(-100.0, 100.0, -100.0, 100.0);
  glMatrixMode(GL_MODELVIEW);

  for (int i = 0; i < num_balls; ++i) {
    Ball b(point2(rand()/static_cast<float>(RAND_MAX) * 200 - 100,
                  rand()/static_cast<float>(RAND_MAX) * 200 - 100),
           color(rand()/static_cast<float>(RAND_MAX),
                 rand()/static_cast<float>(RAND_MAX),
                 rand()/static_cast<float>(RAND_MAX)));
    balls.push_back(b);
  }
}

void DrawLine(const point2& a, const point2& b) {
  glLineWidth(2.0f);
  glBegin(GL_LINES);
  glVertex2f(a.x(), a.y());
  glVertex2f(b.x(), b.y());
  glEnd();
}

void Display() {
  glClear(GL_COLOR_BUFFER_BIT);

  for (int i = 0; i < num_balls; ++i) {
    balls[i].draw();
  }

  ball.draw();
  ball2.draw();

  // if (mouse_cur.x() != mouse_down.x() || mouse_cur.y() != mouse_down.y()) {
  //   glColor3f(1, 1, 1);
  //   DrawLine(win2obj(mouse_down), win2obj(mouse_cur));
  // }

  glFlush();
  glutSwapBuffers();
}

void mouse(int button, int state, int x, int y) {
  if (button == GLUT_LEFT_BUTTON) {
    if (state == GLUT_DOWN) {
      mouse_down = point2(x, window_height-y);
      mouse_cur = mouse_down;
      down = true;
    } else if (state == GLUT_UP) {
      point2 mouse_up = point2(x, window_height-y);
      hit(mouse_up, mouse_down);
      mouse_cur = mouse_down;
    }
  }
}

// glEnable(GL_COLOR_LOGIC_OP);	
// glDrawBuffer(GL_FRONT);
// glLogicOp(GL_XOR);
// glLogicOp(GL_COPY);
// glFlush();
// glDrawBuffer(GL_BACK);
void MouseMotion(int x, int y) {
  glEnable(GL_COLOR_LOGIC_OP);
  glDrawBuffer(GL_FRONT);

  if (down) {
    glLogicOp(GL_XOR);
    glColor3f(1, 1, 1);
    DrawLine(win2obj(mouse_down), win2obj(mouse_cur));

    mouse_cur = point2(x, window_height-y);

    DrawLine(win2obj(mouse_down), win2obj(mouse_cur));
    glLogicOp(GL_COPY);
    glFlush();
  }

  glDrawBuffer(GL_BACK);
}

void keyboard(unsigned char key, int x, int y) {
  switch (key) {
    case 'q':
      exit(EXIT_SUCCESS);
      break;
  }
}

void idle() {
  ball.update();
  ball2.update();

  bool update = !ball.update(&ball2);
  bool update2 = update;

  if (!bb.in(ball.pos(), ball.radius())) {
    const pair<point2, point2> line = bb.closest_line(ball.last_good_pos());
    const vec2 v = reflection(ball.vec(), line.first, line.second);
    ball.set_velocity(v);
    ball.revert_pos();
    update = false;
  }
  if (!bb.in(ball2.pos(), ball2.radius())) {
    const pair<point2, point2> line = bb.closest_line(ball2.last_good_pos());
    const vec2 v = reflection(ball2.vec(), line.first, line.second);
    ball2.set_velocity(v);
    ball2.revert_pos();
    update2 = false;
  }

  if (update) {
    ball.update_last_good_pos();
  }
  if (update2) {
    ball2.update_last_good_pos();
  }

  if (ball.active()) {
    glutPostRedisplay();
  }
}


int main(int argc, char** argv) {
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
  glutInitWindowSize(500, 500);
  glutInitWindowPosition(0, 0);
  glutCreateWindow("Billiards");
  glutDisplayFunc(Display);
  glutKeyboardFunc(keyboard);
  glutMouseFunc(mouse);
  glutMotionFunc(MouseMotion);
  glutIdleFunc(idle);

  const int i = atoi(argv[1]);
  if (i == 1) {
    glutDisplayFunc(Display1);
    glutMotionFunc(MouseMotion1);
  } else if (i == 2) {
    glutDisplayFunc(Display2);
    glutMotionFunc(MouseMotion2);
  } else if (i == 3) {
    glutDisplayFunc(Display3);
    glutMotionFunc(MouseMotion3);
  } else if (i == 4) {
    glutDisplayFunc(Display4);
    glutMotionFunc(MouseMotion4);
  } else if (i == 5) {
    glutDisplayFunc(Display5);
    glutMotionFunc(MouseMotion5);
  } else if (i == 6) {
    glutDisplayFunc(Display6);
    glutMotionFunc(MouseMotion6);
  }


  init();

  glutMainLoop();
}
